package java.math;

import sun.misc.*;
import java.security.*;
import java.util.*;
import java.io.*;

public class BigInteger extends Number implements Comparable<BigInteger>
{
    final int signum;
    final int[] mag;
    private int bitCount;
    private int bitLength;
    private int lowestSetBit;
    private int firstNonzeroIntNum;
    static final long LONG_MASK = 4294967295L;
    private static long[] bitsPerDigit;
    private static final int SMALL_PRIME_THRESHOLD = 95;
    private static final int DEFAULT_PRIME_CERTAINTY = 100;
    private static final BigInteger SMALL_PRIME_PRODUCT;
    private static volatile Random staticRandom;
    private static final int MAX_CONSTANT = 16;
    private static BigInteger[] posConst;
    private static BigInteger[] negConst;
    public static final BigInteger ZERO;
    public static final BigInteger ONE;
    private static final BigInteger TWO;
    public static final BigInteger TEN;
    static int[] bnExpModThreshTable;
    private static String[] zeros;
    private static int[] digitsPerLong;
    private static BigInteger[] longRadix;
    private static int[] digitsPerInt;
    private static int[] intRadix;
    private static final long serialVersionUID = -8287574255936472291L;
    private static final ObjectStreamField[] serialPersistentFields;
    private static final Unsafe unsafe;
    private static final long signumOffset;
    private static final long magOffset;
    
    public BigInteger(final byte[] array) {
        super();
        if (array.length == 0) {
            throw new NumberFormatException("Zero length BigInteger");
        }
        if (array[0] < 0) {
            this.mag = makePositive(array);
            this.signum = -1;
        }
        else {
            this.mag = stripLeadingZeroBytes(array);
            this.signum = ((this.mag.length != 0) ? 1 : 0);
        }
    }
    
    private BigInteger(final int[] array) {
        super();
        if (array.length == 0) {
            throw new NumberFormatException("Zero length BigInteger");
        }
        if (array[0] < 0) {
            this.mag = makePositive(array);
            this.signum = -1;
        }
        else {
            this.mag = trustedStripLeadingZeroInts(array);
            this.signum = ((this.mag.length != 0) ? 1 : 0);
        }
    }
    
    public BigInteger(final int signum, final byte[] array) {
        super();
        this.mag = stripLeadingZeroBytes(array);
        if (signum < -1 || signum > 1) {
            throw new NumberFormatException("Invalid signum value");
        }
        if (this.mag.length == 0) {
            this.signum = 0;
        }
        else {
            if (signum == 0) {
                throw new NumberFormatException("signum-magnitude mismatch");
            }
            this.signum = signum;
        }
    }
    
    private BigInteger(final int signum, final int[] array) {
        super();
        this.mag = stripLeadingZeroInts(array);
        if (signum < -1 || signum > 1) {
            throw new NumberFormatException("Invalid signum value");
        }
        if (this.mag.length == 0) {
            this.signum = 0;
        }
        else {
            if (signum == 0) {
                throw new NumberFormatException("signum-magnitude mismatch");
            }
            this.signum = signum;
        }
    }
    
    public BigInteger(final String s, final int n) {
        super();
        int n2 = 0;
        final int length = s.length();
        if (n < 2 || n > 36) {
            throw new NumberFormatException("Radix out of range");
        }
        if (length == 0) {
            throw new NumberFormatException("Zero length BigInteger");
        }
        int signum = 1;
        final int lastIndex = s.lastIndexOf(45);
        final int lastIndex2 = s.lastIndexOf(43);
        if (lastIndex + lastIndex2 > -1) {
            throw new NumberFormatException("Illegal embedded sign character");
        }
        if (lastIndex == 0 || lastIndex2 == 0) {
            n2 = 1;
            if (length == 1) {
                throw new NumberFormatException("Zero length BigInteger");
            }
        }
        if (lastIndex == 0) {
            signum = -1;
        }
        while (n2 < length && Character.digit(s.charAt(n2), n) == 0) {
            ++n2;
        }
        if (n2 == length) {
            this.signum = 0;
            this.mag = BigInteger.ZERO.mag;
            return;
        }
        final int n3 = length - n2;
        this.signum = signum;
        final int n4 = (int)((n3 * BigInteger.bitsPerDigit[n] >>> 10) + 1L) + 31 >>> 5;
        final int[] array = new int[n4];
        int n5 = n3 % BigInteger.digitsPerInt[n];
        if (n5 == 0) {
            n5 = BigInteger.digitsPerInt[n];
        }
        int i;
        array[n4 - 1] = Integer.parseInt(s.substring(n2, i = n2 + n5), n);
        if (array[n4 - 1] < 0) {
            throw new NumberFormatException("Illegal digit");
        }
        final int n6 = BigInteger.intRadix[n];
        while (i < length) {
            final int parseInt = Integer.parseInt(s.substring(i, i += BigInteger.digitsPerInt[n]), n);
            if (parseInt < 0) {
                throw new NumberFormatException("Illegal digit");
            }
            destructiveMulAdd(array, n6, parseInt);
        }
        this.mag = trustedStripLeadingZeroInts(array);
    }
    
    BigInteger(final char[] array) {
        super();
        int n = 0;
        final int length = array.length;
        int signum = 1;
        if (array[0] == '-') {
            if (length == 1) {
                throw new NumberFormatException("Zero length BigInteger");
            }
            signum = -1;
            n = 1;
        }
        else if (array[0] == '+') {
            if (length == 1) {
                throw new NumberFormatException("Zero length BigInteger");
            }
            n = 1;
        }
        while (n < length && Character.digit(array[n], 10) == 0) {
            ++n;
        }
        if (n == length) {
            this.signum = 0;
            this.mag = BigInteger.ZERO.mag;
            return;
        }
        final int n2 = length - n;
        this.signum = signum;
        int n3;
        if (length < 10) {
            n3 = 1;
        }
        else {
            n3 = (int)((n2 * BigInteger.bitsPerDigit[10] >>> 10) + 1L) + 31 >>> 5;
        }
        final int[] array2 = new int[n3];
        int n4 = n2 % BigInteger.digitsPerInt[10];
        if (n4 == 0) {
            n4 = BigInteger.digitsPerInt[10];
        }
        int i;
        array2[n3 - 1] = this.parseInt(array, n, i = n + n4);
        while (i < length) {
            destructiveMulAdd(array2, BigInteger.intRadix[10], this.parseInt(array, i, i += BigInteger.digitsPerInt[10]));
        }
        this.mag = trustedStripLeadingZeroInts(array2);
    }
    
    private int parseInt(final char[] array, int n, final int n2) {
        int digit = Character.digit(array[n++], 10);
        if (digit == -1) {
            throw new NumberFormatException(new String(array));
        }
        for (int i = n; i < n2; ++i) {
            final int digit2 = Character.digit(array[i], 10);
            if (digit2 == -1) {
                throw new NumberFormatException(new String(array));
            }
            digit = 10 * digit + digit2;
        }
        return digit;
    }
    
    private static void destructiveMulAdd(final int[] array, final int n, final int n2) {
        final long n3 = n & 0xFFFFFFFFL;
        final long n4 = n2 & 0xFFFFFFFFL;
        final int length = array.length;
        long n5 = 0L;
        for (int i = length - 1; i >= 0; --i) {
            final long n6 = n3 * (array[i] & 0xFFFFFFFFL) + n5;
            array[i] = (int)n6;
            n5 = n6 >>> 32;
        }
        final long n7 = (array[length - 1] & 0xFFFFFFFFL) + n4;
        array[length - 1] = (int)n7;
        long n8 = n7 >>> 32;
        for (int j = length - 2; j >= 0; --j) {
            final long n9 = (array[j] & 0xFFFFFFFFL) + n8;
            array[j] = (int)n9;
            n8 = n9 >>> 32;
        }
    }
    
    public BigInteger(final String s) {
        this(s, 10);
    }
    
    public BigInteger(final int n, final Random random) {
        this(1, randomBits(n, random));
    }
    
    private static byte[] randomBits(final int n, final Random random) {
        if (n < 0) {
            throw new IllegalArgumentException("numBits must be non-negative");
        }
        final int n2 = (int)((n + 7L) / 8L);
        final byte[] array = new byte[n2];
        if (n2 > 0) {
            random.nextBytes(array);
            final int n3 = 8 * n2 - n;
            final byte[] array2 = array;
            final int n4 = 0;
            array2[n4] &= (byte)((1 << 8 - n3) - 1);
        }
        return array;
    }
    
    public BigInteger(final int n, final int n2, final Random random) {
        super();
        if (n < 2) {
            throw new ArithmeticException("bitLength < 2");
        }
        final BigInteger bigInteger = (n < 95) ? smallPrime(n, n2, random) : largePrime(n, n2, random);
        this.signum = 1;
        this.mag = bigInteger.mag;
    }
    
    public static BigInteger probablePrime(final int n, final Random random) {
        if (n < 2) {
            throw new ArithmeticException("bitLength < 2");
        }
        return (n < 95) ? smallPrime(n, 100, random) : largePrime(n, 100, random);
    }
    
    private static BigInteger smallPrime(final int n, final int n2, final Random random) {
        final int n3 = n + 31 >>> 5;
        final int[] array = new int[n3];
        final int n4 = 1 << (n + 31 & 0x1F);
        final int n5 = (n4 << 1) - 1;
        while (true) {
            for (int i = 0; i < n3; ++i) {
                array[i] = random.nextInt();
            }
            array[0] = ((array[0] & n5) | n4);
            if (n > 2) {
                final int[] array2 = array;
                final int n6 = n3 - 1;
                array2[n6] |= 0x1;
            }
            final BigInteger bigInteger = new BigInteger(array, 1);
            if (n > 6) {
                final long longValue = bigInteger.remainder(BigInteger.SMALL_PRIME_PRODUCT).longValue();
                if (longValue % 3L == 0L || longValue % 5L == 0L || longValue % 7L == 0L || longValue % 11L == 0L || longValue % 13L == 0L || longValue % 17L == 0L || longValue % 19L == 0L || longValue % 23L == 0L || longValue % 29L == 0L || longValue % 31L == 0L || longValue % 37L == 0L) {
                    continue;
                }
                if (longValue % 41L == 0L) {
                    continue;
                }
            }
            if (n < 4) {
                return bigInteger;
            }
            if (bigInteger.primeToCertainty(n2, random)) {
                return bigInteger;
            }
        }
    }
    
    private static BigInteger largePrime(final int n, final int n2, final Random random) {
        BigInteger bigInteger = new BigInteger(n, random).setBit(n - 1);
        final int[] mag = bigInteger.mag;
        final int n3 = bigInteger.mag.length - 1;
        mag[n3] &= 0xFFFFFFFE;
        int n4;
        BigInteger bigInteger2;
        int[] mag2;
        int n5;
        for (n4 = n / 20 * 64, bigInteger2 = new BitSieve(bigInteger, n4).retrieve(bigInteger, n2, random); bigInteger2 == null || bigInteger2.bitLength() != n; bigInteger2 = new BitSieve(bigInteger, n4).retrieve(bigInteger, n2, random)) {
            bigInteger = bigInteger.add(valueOf(2 * n4));
            if (bigInteger.bitLength() != n) {
                bigInteger = new BigInteger(n, random).setBit(n - 1);
            }
            mag2 = bigInteger.mag;
            n5 = bigInteger.mag.length - 1;
            mag2[n5] &= 0xFFFFFFFE;
        }
        return bigInteger2;
    }
    
    public BigInteger nextProbablePrime() {
        if (this.signum < 0) {
            throw new ArithmeticException("start < 0: " + this);
        }
        if (this.signum == 0 || this.equals(BigInteger.ONE)) {
            return BigInteger.TWO;
        }
        BigInteger bigInteger = this.add(BigInteger.ONE);
        if (bigInteger.bitLength() >= 95) {
            if (bigInteger.testBit(0)) {
                bigInteger = bigInteger.subtract(BigInteger.ONE);
            }
            final int n = bigInteger.bitLength() / 20 * 64;
            BigInteger retrieve;
            while (true) {
                retrieve = new BitSieve(bigInteger, n).retrieve(bigInteger, 100, null);
                if (retrieve != null) {
                    break;
                }
                bigInteger = bigInteger.add(valueOf(2 * n));
            }
            return retrieve;
        }
        if (!bigInteger.testBit(0)) {
            bigInteger = bigInteger.add(BigInteger.ONE);
        }
        while (true) {
            if (bigInteger.bitLength() > 6) {
                final long longValue = bigInteger.remainder(BigInteger.SMALL_PRIME_PRODUCT).longValue();
                if (longValue % 3L == 0L || longValue % 5L == 0L || longValue % 7L == 0L || longValue % 11L == 0L || longValue % 13L == 0L || longValue % 17L == 0L || longValue % 19L == 0L || longValue % 23L == 0L || longValue % 29L == 0L || longValue % 31L == 0L || longValue % 37L == 0L || longValue % 41L == 0L) {
                    bigInteger = bigInteger.add(BigInteger.TWO);
                    continue;
                }
            }
            if (bigInteger.bitLength() < 4) {
                return bigInteger;
            }
            if (bigInteger.primeToCertainty(100, null)) {
                return bigInteger;
            }
            bigInteger = bigInteger.add(BigInteger.TWO);
        }
    }
    
    boolean primeToCertainty(final int n, final Random random) {
        final int n2 = (Math.min(n, 2147483646) + 1) / 2;
        final int bitLength = this.bitLength();
        if (bitLength < 100) {
            final int n3 = 50;
            return this.passesMillerRabin((n2 < n3) ? n2 : n3, random);
        }
        int n4;
        if (bitLength < 256) {
            n4 = 27;
        }
        else if (bitLength < 512) {
            n4 = 15;
        }
        else if (bitLength < 768) {
            n4 = 8;
        }
        else if (bitLength < 1024) {
            n4 = 4;
        }
        else {
            n4 = 2;
        }
        return this.passesMillerRabin((n2 < n4) ? n2 : n4, random) && this.passesLucasLehmer();
    }
    
    private boolean passesLucasLehmer() {
        final BigInteger add = this.add(BigInteger.ONE);
        int n;
        for (n = 5; jacobiSymbol(n, this) != -1; n = ((n < 0) ? (Math.abs(n) + 2) : (-(n + 2)))) {}
        return lucasLehmerSequence(n, add, this).mod(this).equals(BigInteger.ZERO);
    }
    
    private static int jacobiSymbol(int n, final BigInteger bigInteger) {
        if (n == 0) {
            return 0;
        }
        int n2 = 1;
        final int n3 = bigInteger.mag[bigInteger.mag.length - 1];
        if (n < 0) {
            n = -n;
            final int n4 = n3 & 0x7;
            if (n4 == 3 || n4 == 7) {
                n2 = -n2;
            }
        }
        while ((n & 0x3) == 0x0) {
            n >>= 2;
        }
        if ((n & 0x1) == 0x0) {
            n >>= 1;
            if (((n3 ^ n3 >> 1) & 0x2) != 0x0) {
                n2 = -n2;
            }
        }
        if (n == 1) {
            return n2;
        }
        if ((n & n3 & 0x2) != 0x0) {
            n2 = -n2;
        }
        int n6;
        for (int i = bigInteger.mod(valueOf(n)).intValue(); i != 0; i = n6 % n) {
            while ((i & 0x3) == 0x0) {
                i >>= 2;
            }
            if ((i & 0x1) == 0x0) {
                i >>= 1;
                if (((n ^ n >> 1) & 0x2) != 0x0) {
                    n2 = -n2;
                }
            }
            if (i == 1) {
                return n2;
            }
            assert i < n;
            final int n5 = i;
            n6 = n;
            n = n5;
            if ((n6 & n & 0x2) != 0x0) {
                n2 = -n2;
            }
        }
        return 0;
    }
    
    private static BigInteger lucasLehmerSequence(final int n, final BigInteger bigInteger, final BigInteger bigInteger2) {
        final BigInteger value = valueOf(n);
        BigInteger one = BigInteger.ONE;
        BigInteger one2 = BigInteger.ONE;
        for (int i = bigInteger.bitLength() - 2; i >= 0; --i) {
            final BigInteger mod = one.multiply(one2).mod(bigInteger2);
            BigInteger bigInteger3 = one2.square().add(value.multiply(one.square())).mod(bigInteger2);
            if (bigInteger3.testBit(0)) {
                bigInteger3 = bigInteger3.subtract(bigInteger2);
            }
            final BigInteger shiftRight = bigInteger3.shiftRight(1);
            one = mod;
            one2 = shiftRight;
            if (bigInteger.testBit(i)) {
                BigInteger bigInteger4 = one.add(one2).mod(bigInteger2);
                if (bigInteger4.testBit(0)) {
                    bigInteger4 = bigInteger4.subtract(bigInteger2);
                }
                final BigInteger shiftRight2 = bigInteger4.shiftRight(1);
                BigInteger bigInteger5 = one2.add(value.multiply(one)).mod(bigInteger2);
                if (bigInteger5.testBit(0)) {
                    bigInteger5 = bigInteger5.subtract(bigInteger2);
                }
                final BigInteger shiftRight3 = bigInteger5.shiftRight(1);
                one = shiftRight2;
                one2 = shiftRight3;
            }
        }
        return one;
    }
    
    private static Random getSecureRandom() {
        if (BigInteger.staticRandom == null) {
            BigInteger.staticRandom = new SecureRandom();
        }
        return BigInteger.staticRandom;
    }
    
    private boolean passesMillerRabin(final int n, Random secureRandom) {
        final BigInteger subtract;
        final BigInteger bigInteger = subtract = this.subtract(BigInteger.ONE);
        final int lowestSetBit = subtract.getLowestSetBit();
        final BigInteger shiftRight = subtract.shiftRight(lowestSetBit);
        if (secureRandom == null) {
            secureRandom = getSecureRandom();
        }
        for (int i = 0; i < n; ++i) {
            BigInteger bigInteger2;
            do {
                bigInteger2 = new BigInteger(this.bitLength(), secureRandom);
            } while (bigInteger2.compareTo(BigInteger.ONE) <= 0 || bigInteger2.compareTo(this) >= 0);
            int n2 = 0;
            for (BigInteger bigInteger3 = bigInteger2.modPow(shiftRight, this); (n2 != 0 || !bigInteger3.equals(BigInteger.ONE)) && !bigInteger3.equals(bigInteger); bigInteger3 = bigInteger3.modPow(BigInteger.TWO, this)) {
                if ((n2 > 0 && bigInteger3.equals(BigInteger.ONE)) || ++n2 == lowestSetBit) {
                    return false;
                }
            }
        }
        return true;
    }
    
    BigInteger(final int[] mag, final int n) {
        super();
        this.signum = ((mag.length == 0) ? 0 : n);
        this.mag = mag;
    }
    
    private BigInteger(final byte[] array, final int n) {
        super();
        this.signum = ((array.length == 0) ? 0 : n);
        this.mag = stripLeadingZeroBytes(array);
    }
    
    public static BigInteger valueOf(final long n) {
        if (n == 0L) {
            return BigInteger.ZERO;
        }
        if (n > 0L && n <= 16L) {
            return BigInteger.posConst[(int)n];
        }
        if (n < 0L && n >= -16L) {
            return BigInteger.negConst[(int)(-n)];
        }
        return new BigInteger(n);
    }
    
    private BigInteger(long n) {
        super();
        if (n < 0L) {
            n = -n;
            this.signum = -1;
        }
        else {
            this.signum = 1;
        }
        final int n2 = (int)(n >>> 32);
        if (n2 == 0) {
            (this.mag = new int[1])[0] = (int)n;
        }
        else {
            (this.mag = new int[2])[0] = n2;
            this.mag[1] = (int)n;
        }
    }
    
    private static BigInteger valueOf(final int[] array) {
        return (array[0] > 0) ? new BigInteger(array, 1) : new BigInteger(array);
    }
    
    public BigInteger add(final BigInteger bigInteger) {
        if (bigInteger.signum == 0) {
            return this;
        }
        if (this.signum == 0) {
            return bigInteger;
        }
        if (bigInteger.signum == this.signum) {
            return new BigInteger(add(this.mag, bigInteger.mag), this.signum);
        }
        final int compareMagnitude = this.compareMagnitude(bigInteger);
        if (compareMagnitude == 0) {
            return BigInteger.ZERO;
        }
        return new BigInteger(trustedStripLeadingZeroInts((compareMagnitude > 0) ? subtract(this.mag, bigInteger.mag) : subtract(bigInteger.mag, this.mag)), (compareMagnitude == this.signum) ? 1 : -1);
    }
    
    private static int[] add(int[] array, int[] array2) {
        if (array.length < array2.length) {
            final int[] array3 = array;
            array = array2;
            array2 = array3;
        }
        int i;
        int j;
        int[] array4;
        long n;
        for (i = array.length, j = array2.length, array4 = new int[i], n = 0L; j > 0; n = (array[--i] & 0xFFFFFFFFL) + (array2[--j] & 0xFFFFFFFFL) + (n >>> 32), array4[i] = (int)n) {}
        boolean b;
        for (b = (n >>> 32 != 0L); i > 0 && b; b = ((array4[--i] = array[i] + 1) == 0)) {}
        while (i > 0) {
            array4[--i] = array[i];
        }
        if (b) {
            final int[] array5 = new int[array4.length + 1];
            System.arraycopy(array4, 0, array5, 1, array4.length);
            array5[0] = 1;
            return array5;
        }
        return array4;
    }
    
    public BigInteger subtract(final BigInteger bigInteger) {
        if (bigInteger.signum == 0) {
            return this;
        }
        if (this.signum == 0) {
            return bigInteger.negate();
        }
        if (bigInteger.signum != this.signum) {
            return new BigInteger(add(this.mag, bigInteger.mag), this.signum);
        }
        final int compareMagnitude = this.compareMagnitude(bigInteger);
        if (compareMagnitude == 0) {
            return BigInteger.ZERO;
        }
        return new BigInteger(trustedStripLeadingZeroInts((compareMagnitude > 0) ? subtract(this.mag, bigInteger.mag) : subtract(bigInteger.mag, this.mag)), (compareMagnitude == this.signum) ? 1 : -1);
    }
    
    private static int[] subtract(final int[] array, final int[] array2) {
        int i;
        int[] array3;
        int j;
        long n;
        for (i = array.length, array3 = new int[i], j = array2.length, n = 0L; j > 0; n = (array[--i] & 0xFFFFFFFFL) - (array2[--j] & 0xFFFFFFFFL) + (n >> 32), array3[i] = (int)n) {}
        for (boolean b = n >> 32 != 0L; i > 0 && b; b = ((array3[--i] = array[i] - 1) == -1)) {}
        while (i > 0) {
            array3[--i] = array[i];
        }
        return array3;
    }
    
    public BigInteger multiply(final BigInteger bigInteger) {
        if (bigInteger.signum == 0 || this.signum == 0) {
            return BigInteger.ZERO;
        }
        return new BigInteger(trustedStripLeadingZeroInts(this.multiplyToLen(this.mag, this.mag.length, bigInteger.mag, bigInteger.mag.length, null)), (this.signum == bigInteger.signum) ? 1 : -1);
    }
    
    BigInteger multiply(long n) {
        if (n == 0L || this.signum == 0) {
            return BigInteger.ZERO;
        }
        if (n == Long.MIN_VALUE) {
            return this.multiply(valueOf(n));
        }
        final int n2 = (n > 0L) ? this.signum : (-this.signum);
        if (n < 0L) {
            n = -n;
        }
        final long n3 = n >>> 32;
        final long n4 = n & 0xFFFFFFFFL;
        final int length = this.mag.length;
        final int[] mag = this.mag;
        int[] copyOfRange = (n3 == 0L) ? new int[length + 1] : new int[length + 2];
        long n5 = 0L;
        int n6 = copyOfRange.length - 1;
        for (int i = length - 1; i >= 0; --i) {
            final long n7 = (mag[i] & 0xFFFFFFFFL) * n4 + n5;
            copyOfRange[n6--] = (int)n7;
            n5 = n7 >>> 32;
        }
        copyOfRange[n6] = (int)n5;
        if (n3 != 0L) {
            n5 = 0L;
            int n8 = copyOfRange.length - 2;
            for (int j = length - 1; j >= 0; --j) {
                final long n9 = (mag[j] & 0xFFFFFFFFL) * n3 + (copyOfRange[n8] & 0xFFFFFFFFL) + n5;
                copyOfRange[n8--] = (int)n9;
                n5 = n9 >>> 32;
            }
            copyOfRange[0] = (int)n5;
        }
        if (n5 == 0L) {
            copyOfRange = Arrays.copyOfRange(copyOfRange, 1, copyOfRange.length);
        }
        return new BigInteger(copyOfRange, n2);
    }
    
    private int[] multiplyToLen(final int[] array, final int n, final int[] array2, final int n2, int[] array3) {
        final int n3 = n - 1;
        final int n4 = n2 - 1;
        if (array3 == null || array3.length < n + n2) {
            array3 = new int[n + n2];
        }
        long n5 = 0L;
        for (int i = n4, n6 = n4 + 1 + n3; i >= 0; --i, --n6) {
            final long n7 = (array2[i] & 0xFFFFFFFFL) * (array[n3] & 0xFFFFFFFFL) + n5;
            array3[n6] = (int)n7;
            n5 = n7 >>> 32;
        }
        array3[n3] = (int)n5;
        for (int j = n3 - 1; j >= 0; --j) {
            long n8 = 0L;
            for (int k = n4, n9 = n4 + 1 + j; k >= 0; --k, --n9) {
                final long n10 = (array2[k] & 0xFFFFFFFFL) * (array[j] & 0xFFFFFFFFL) + (array3[n9] & 0xFFFFFFFFL) + n8;
                array3[n9] = (int)n10;
                n8 = n10 >>> 32;
            }
            array3[j] = (int)n8;
        }
        return array3;
    }
    
    private BigInteger square() {
        if (this.signum == 0) {
            return BigInteger.ZERO;
        }
        return new BigInteger(trustedStripLeadingZeroInts(squareToLen(this.mag, this.mag.length, null)), 1);
    }
    
    private static final int[] squareToLen(final int[] array, final int n, int[] array2) {
        final int n2 = n << 1;
        if (array2 == null || array2.length < n2) {
            array2 = new int[n2];
        }
        int n3 = 0;
        int i = 0;
        int n4 = 0;
        while (i < n) {
            final long n5 = array[i] & 0xFFFFFFFFL;
            final long n6 = n5 * n5;
            array2[n4++] = (n3 << 31 | (int)(n6 >>> 33));
            array2[n4++] = (int)(n6 >>> 1);
            n3 = (int)n6;
            ++i;
        }
        for (int j = n, n7 = 1; j > 0; --j, n7 += 2) {
            addOne(array2, n7 - 1, j, mulAdd(array2, array, n7, j - 1, array[j - 1]));
        }
        primitiveLeftShift(array2, n2, 1);
        final int[] array3 = array2;
        final int n8 = n2 - 1;
        array3[n8] |= (array[n - 1] & 0x1);
        return array2;
    }
    
    public BigInteger divide(final BigInteger bigInteger) {
        final MutableBigInteger mutableBigInteger = new MutableBigInteger();
        new MutableBigInteger(this.mag).divide(new MutableBigInteger(bigInteger.mag), mutableBigInteger);
        return mutableBigInteger.toBigInteger((this.signum == bigInteger.signum) ? 1 : -1);
    }
    
    public BigInteger[] divideAndRemainder(final BigInteger bigInteger) {
        final BigInteger[] array = new BigInteger[2];
        final MutableBigInteger mutableBigInteger = new MutableBigInteger();
        final MutableBigInteger divide = new MutableBigInteger(this.mag).divide(new MutableBigInteger(bigInteger.mag), mutableBigInteger);
        array[0] = mutableBigInteger.toBigInteger((this.signum == bigInteger.signum) ? 1 : -1);
        array[1] = divide.toBigInteger(this.signum);
        return array;
    }
    
    public BigInteger remainder(final BigInteger bigInteger) {
        return new MutableBigInteger(this.mag).divide(new MutableBigInteger(bigInteger.mag), new MutableBigInteger()).toBigInteger(this.signum);
    }
    
    public BigInteger pow(int i) {
        if (i < 0) {
            throw new ArithmeticException("Negative exponent");
        }
        if (this.signum == 0) {
            return (i == 0) ? BigInteger.ONE : this;
        }
        final int n = (this.signum < 0 && (i & 0x1) == 0x1) ? -1 : 1;
        int[] array = this.mag;
        int[] trustedStripLeadingZeroInts = { 1 };
        while (i != 0) {
            if ((i & 0x1) == 0x1) {
                trustedStripLeadingZeroInts = trustedStripLeadingZeroInts(this.multiplyToLen(trustedStripLeadingZeroInts, trustedStripLeadingZeroInts.length, array, array.length, null));
            }
            if ((i >>>= 1) == 0) {
                continue;
            }
            array = trustedStripLeadingZeroInts(squareToLen(array, array.length, null));
        }
        return new BigInteger(trustedStripLeadingZeroInts, n);
    }
    
    public BigInteger gcd(final BigInteger bigInteger) {
        if (bigInteger.signum == 0) {
            return this.abs();
        }
        if (this.signum == 0) {
            return bigInteger.abs();
        }
        return new MutableBigInteger(this).hybridGCD(new MutableBigInteger(bigInteger)).toBigInteger(1);
    }
    
    static int bitLengthForInt(final int n) {
        return 32 - Integer.numberOfLeadingZeros(n);
    }
    
    private static int[] leftShift(final int[] array, final int n, final int n2) {
        final int n3 = n2 >>> 5;
        final int n4 = n2 & 0x1F;
        final int bitLengthForInt = bitLengthForInt(array[0]);
        if (n2 <= 32 - bitLengthForInt) {
            primitiveLeftShift(array, n, n4);
            return array;
        }
        if (n4 <= 32 - bitLengthForInt) {
            final int[] array2 = new int[n3 + n];
            for (int i = 0; i < n; ++i) {
                array2[i] = array[i];
            }
            primitiveLeftShift(array2, array2.length, n4);
            return array2;
        }
        final int[] array3 = new int[n3 + n + 1];
        for (int j = 0; j < n; ++j) {
            array3[j] = array[j];
        }
        primitiveRightShift(array3, array3.length, 32 - n4);
        return array3;
    }
    
    static void primitiveRightShift(final int[] array, final int n, final int n2) {
        final int n3 = 32 - n2;
        int i = n - 1;
        int n4 = array[i];
        while (i > 0) {
            final int n5 = n4;
            n4 = array[i - 1];
            array[i] = (n4 << n3 | n5 >>> n2);
            --i;
        }
        final int n6 = 0;
        array[n6] >>>= n2;
    }
    
    static void primitiveLeftShift(final int[] array, final int n, final int n2) {
        if (n == 0 || n2 == 0) {
            return;
        }
        final int n3 = 32 - n2;
        int i = 0;
        int n4 = array[i];
        while (i < i + n - 1) {
            final int n5 = n4;
            n4 = array[i + 1];
            array[i] = (n5 << n2 | n4 >>> n3);
            ++i;
        }
        final int n6 = n - 1;
        array[n6] <<= n2;
    }
    
    private static int bitLength(final int[] array, final int n) {
        if (n == 0) {
            return 0;
        }
        return (n - 1 << 5) + bitLengthForInt(array[0]);
    }
    
    public BigInteger abs() {
        return (this.signum >= 0) ? this : this.negate();
    }
    
    public BigInteger negate() {
        return new BigInteger(this.mag, -this.signum);
    }
    
    public int signum() {
        return this.signum;
    }
    
    public BigInteger mod(final BigInteger bigInteger) {
        if (bigInteger.signum <= 0) {
            throw new ArithmeticException("BigInteger: modulus not positive");
        }
        final BigInteger remainder = this.remainder(bigInteger);
        return (remainder.signum >= 0) ? remainder : remainder.add(bigInteger);
    }
    
    public BigInteger modPow(BigInteger negate, final BigInteger bigInteger) {
        if (bigInteger.signum <= 0) {
            throw new ArithmeticException("BigInteger: modulus not positive");
        }
        if (negate.signum == 0) {
            return bigInteger.equals(BigInteger.ONE) ? BigInteger.ZERO : BigInteger.ONE;
        }
        if (this.equals(BigInteger.ONE)) {
            return bigInteger.equals(BigInteger.ONE) ? BigInteger.ZERO : BigInteger.ONE;
        }
        if (this.equals(BigInteger.ZERO) && negate.signum >= 0) {
            return BigInteger.ZERO;
        }
        if (this.equals(BigInteger.negConst[1]) && !negate.testBit(0)) {
            return bigInteger.equals(BigInteger.ONE) ? BigInteger.ZERO : BigInteger.ONE;
        }
        final boolean b;
        if (b = (negate.signum < 0)) {
            negate = negate.negate();
        }
        final BigInteger bigInteger2 = (this.signum < 0 || this.compareTo(bigInteger) >= 0) ? this.mod(bigInteger) : this;
        BigInteger bigInteger3;
        if (bigInteger.testBit(0)) {
            bigInteger3 = bigInteger2.oddModPow(negate, bigInteger);
        }
        else {
            final int lowestSetBit = bigInteger.getLowestSetBit();
            final BigInteger shiftRight = bigInteger.shiftRight(lowestSetBit);
            final BigInteger shiftLeft = BigInteger.ONE.shiftLeft(lowestSetBit);
            final BigInteger bigInteger4 = (this.signum < 0 || this.compareTo(shiftRight) >= 0) ? this.mod(shiftRight) : this;
            bigInteger3 = (shiftRight.equals(BigInteger.ONE) ? BigInteger.ZERO : bigInteger4.oddModPow(negate, shiftRight)).multiply(shiftLeft).multiply(shiftLeft.modInverse(shiftRight)).add(bigInteger2.modPow2(negate, lowestSetBit).multiply(shiftRight).multiply(shiftRight.modInverse(shiftLeft))).mod(bigInteger);
        }
        return b ? bigInteger3.modInverse(bigInteger) : bigInteger3;
    }
    
    private BigInteger oddModPow(final BigInteger bigInteger, final BigInteger bigInteger2) {
        if (bigInteger.equals(BigInteger.ONE)) {
            return this;
        }
        if (this.signum == 0) {
            return BigInteger.ZERO;
        }
        final int[] array = this.mag.clone();
        final int[] mag = bigInteger.mag;
        final int[] mag2 = bigInteger2.mag;
        final int length = mag2.length;
        int n = 0;
        int i = bitLength(mag, mag.length);
        if (i != 17 || mag[0] != 65537) {
            while (i > BigInteger.bnExpModThreshTable[n]) {
                ++n;
            }
        }
        final int n2 = 1 << n;
        final int[][] array2 = new int[n2][];
        for (int j = 0; j < n2; ++j) {
            array2[j] = new int[length];
        }
        final int n3 = -MutableBigInteger.inverseMod32(mag2[length - 1]);
        int[] leftShift = leftShift(array, array.length, length << 5);
        array2[0] = new MutableBigInteger(leftShift).divide(new MutableBigInteger(mag2), new MutableBigInteger()).toIntArray();
        if (array2[0].length < length) {
            final int n4 = length - array2[0].length;
            final int[] array3 = new int[length];
            for (int k = 0; k < array2[0].length; ++k) {
                array3[k + n4] = array2[0][k];
            }
            array2[0] = array3;
        }
        int[] montReduce = montReduce(squareToLen(array2[0], length, null), mag2, length, n3);
        final int[] array4 = new int[length];
        for (int l = 0; l < length; ++l) {
            array4[l] = montReduce[l];
        }
        for (int n5 = 1; n5 < n2; ++n5) {
            array2[n5] = montReduce(this.multiplyToLen(array4, length, array2[n5 - 1], length, null), mag2, length, n3);
        }
        int n6 = 1 << (i - 1 & 0x1F);
        int n7 = 0;
        int length2 = mag.length;
        int n8 = 0;
        for (int n9 = 0; n9 <= n; ++n9) {
            n7 = (n7 << 1 | (mag[n8] & n6) != 0x0);
            n6 >>>= 1;
            if (n6 == 0) {
                ++n8;
                n6 = Integer.MIN_VALUE;
                --length2;
            }
        }
        --i;
        int n10 = 1;
        int n11;
        for (n11 = i - n; (n7 & 0x1) == 0x0; n7 >>>= 1, ++n11) {}
        int[] array5 = array2[n7 >>> 1];
        int n12 = 0;
        if (n11 == i) {
            n10 = 0;
        }
        while (true) {
            --i;
            n12 <<= 1;
            if (length2 != 0) {
                n12 |= (((mag[n8] & n6) != 0x0) ? 1 : 0);
                n6 >>>= 1;
                if (n6 == 0) {
                    ++n8;
                    n6 = Integer.MIN_VALUE;
                    --length2;
                }
            }
            if (n12 & n2) {
                for (n11 = i - n; !(n12 & true); n12 >>>= 1, ++n11) {}
                array5 = array2[n12 >>> 1];
                n12 = 0;
            }
            if (i == n11) {
                if (n10 != 0) {
                    montReduce = array5.clone();
                    n10 = 0;
                }
                else {
                    final int[] montReduce2 = montReduce(this.multiplyToLen(montReduce, length, array5, length, leftShift), mag2, length, n3);
                    leftShift = montReduce;
                    montReduce = montReduce2;
                }
            }
            if (i == 0) {
                break;
            }
            if (n10 != 0) {
                continue;
            }
            final int[] montReduce3 = montReduce(squareToLen(montReduce, length, leftShift), mag2, length, n3);
            leftShift = montReduce;
            montReduce = montReduce3;
        }
        final int[] array6 = new int[2 * length];
        for (int n13 = 0; n13 < length; ++n13) {
            array6[n13 + length] = montReduce[n13];
        }
        final int[] montReduce4 = montReduce(array6, mag2, length, n3);
        final int[] array7 = new int[length];
        for (int n14 = 0; n14 < length; ++n14) {
            array7[n14] = montReduce4[n14];
        }
        return new BigInteger(1, array7);
    }
    
    private static int[] montReduce(final int[] array, final int[] array2, int n, final int n2) {
        int i = 0;
        int n3 = 0;
        do {
            i += addOne(array, n3, n, mulAdd(array, array2, n3, n, n2 * array[array.length - 1 - n3]));
            ++n3;
        } while (--n > 0);
        while (i > 0) {
            i += subN(array, array2, n);
        }
        while (intArrayCmpToLen(array, array2, n) >= 0) {
            subN(array, array2, n);
        }
        return array;
    }
    
    private static int intArrayCmpToLen(final int[] array, final int[] array2, final int n) {
        for (int i = 0; i < n; ++i) {
            final long n2 = array[i] & 0xFFFFFFFFL;
            final long n3 = array2[i] & 0xFFFFFFFFL;
            if (n2 < n3) {
                return -1;
            }
            if (n2 > n3) {
                return 1;
            }
        }
        return 0;
    }
    
    private static int subN(final int[] array, final int[] array2, int n) {
        long n2 = 0L;
        while (--n >= 0) {
            n2 = (array[n] & 0xFFFFFFFFL) - (array2[n] & 0xFFFFFFFFL) + (n2 >> 32);
            array[n] = (int)n2;
        }
        return (int)(n2 >> 32);
    }
    
    static int mulAdd(final int[] array, final int[] array2, int n, final int n2, final int n3) {
        final long n4 = n3 & 0xFFFFFFFFL;
        long n5 = 0L;
        n = array.length - n - 1;
        for (int i = n2 - 1; i >= 0; --i) {
            final long n6 = (array2[i] & 0xFFFFFFFFL) * n4 + (array[n] & 0xFFFFFFFFL) + n5;
            array[n--] = (int)n6;
            n5 = n6 >>> 32;
        }
        return (int)n5;
    }
    
    static int addOne(final int[] array, int n, int n2, final int n3) {
        n = array.length - 1 - n2 - n;
        final long n4 = (array[n] & 0xFFFFFFFFL) + (n3 & 0xFFFFFFFFL);
        array[n] = (int)n4;
        if (n4 >>> 32 == 0L) {
            return 0;
        }
        while (--n2 >= 0) {
            if (--n < 0) {
                return 1;
            }
            final int n5 = n;
            ++array[n5];
            if (array[n] != 0) {
                return 0;
            }
        }
        return 1;
    }
    
    private BigInteger modPow2(final BigInteger bigInteger, final int n) {
        BigInteger bigInteger2 = valueOf(1L);
        BigInteger bigInteger3 = this.mod2(n);
        int i = 0;
        int bitLength = bigInteger.bitLength();
        if (this.testBit(0)) {
            bitLength = ((n - 1 < bitLength) ? (n - 1) : bitLength);
        }
        while (i < bitLength) {
            if (bigInteger.testBit(i)) {
                bigInteger2 = bigInteger2.multiply(bigInteger3).mod2(n);
            }
            if (++i >= bitLength) {
                continue;
            }
            bigInteger3 = bigInteger3.square().mod2(n);
        }
        return bigInteger2;
    }
    
    private BigInteger mod2(final int n) {
        if (this.bitLength() <= n) {
            return this;
        }
        final int n2 = n + 31 >>> 5;
        final int[] array = new int[n2];
        for (int i = 0; i < n2; ++i) {
            array[i] = this.mag[i + (this.mag.length - n2)];
        }
        final int n3 = (n2 << 5) - n;
        final int[] array2 = array;
        final int n4 = 0;
        array2[n4] &= (int)((1L << 32 - n3) - 1L);
        return (array[0] == 0) ? new BigInteger(1, array) : new BigInteger(array, 1);
    }
    
    public BigInteger modInverse(final BigInteger bigInteger) {
        if (bigInteger.signum != 1) {
            throw new ArithmeticException("BigInteger: modulus not positive");
        }
        if (bigInteger.equals(BigInteger.ONE)) {
            return BigInteger.ZERO;
        }
        BigInteger mod = this;
        if (this.signum < 0 || this.compareMagnitude(bigInteger) >= 0) {
            mod = this.mod(bigInteger);
        }
        if (mod.equals(BigInteger.ONE)) {
            return BigInteger.ONE;
        }
        return new MutableBigInteger(mod).mutableModInverse(new MutableBigInteger(bigInteger)).toBigInteger(1);
    }
    
    public BigInteger shiftLeft(final int n) {
        if (this.signum == 0) {
            return BigInteger.ZERO;
        }
        if (n == 0) {
            return this;
        }
        if (n >= 0) {
            final int n2 = n >>> 5;
            final int n3 = n & 0x1F;
            final int length = this.mag.length;
            int[] array;
            if (n3 == 0) {
                array = new int[length + n2];
                for (int i = 0; i < length; ++i) {
                    array[i] = this.mag[i];
                }
            }
            else {
                int n4 = 0;
                final int n5 = 32 - n3;
                final int n6 = this.mag[0] >>> n5;
                if (n6 != 0) {
                    array = new int[length + n2 + 1];
                    array[n4++] = n6;
                }
                else {
                    array = new int[length + n2];
                }
                int j;
                for (j = 0; j < length - 1; array[n4++] = (this.mag[j++] << n3 | this.mag[j] >>> n5)) {}
                array[n4] = this.mag[j] << n3;
            }
            return new BigInteger(array, this.signum);
        }
        if (n == Integer.MIN_VALUE) {
            throw new ArithmeticException("Shift distance of Integer.MIN_VALUE not supported.");
        }
        return this.shiftRight(-n);
    }
    
    public BigInteger shiftRight(final int n) {
        if (n == 0) {
            return this;
        }
        if (n < 0) {
            if (n == Integer.MIN_VALUE) {
                throw new ArithmeticException("Shift distance of Integer.MIN_VALUE not supported.");
            }
            return this.shiftLeft(-n);
        }
        else {
            final int n2 = n >>> 5;
            final int n3 = n & 0x1F;
            final int length = this.mag.length;
            if (n2 >= length) {
                return (this.signum >= 0) ? BigInteger.ZERO : BigInteger.negConst[1];
            }
            int[] javaIncrement;
            if (n3 == 0) {
                final int n4 = length - n2;
                javaIncrement = new int[n4];
                for (int i = 0; i < n4; ++i) {
                    javaIncrement[i] = this.mag[i];
                }
            }
            else {
                int n5 = 0;
                final int n6 = this.mag[0] >>> n3;
                if (n6 != 0) {
                    javaIncrement = new int[length - n2];
                    javaIncrement[n5++] = n6;
                }
                else {
                    javaIncrement = new int[length - n2 - 1];
                }
                for (int n7 = 32 - n3, j = 0; j < length - n2 - 1; javaIncrement[n5++] = (this.mag[j++] << n7 | this.mag[j] >>> n3)) {}
            }
            if (this.signum < 0) {
                boolean b = false;
                for (int n8 = length - 1; n8 >= length - n2 && !b; b = (this.mag[n8] != 0), --n8) {}
                if (!b && n3 != 0) {
                    b = (this.mag[length - n2 - 1] << 32 - n3 != 0);
                }
                if (b) {
                    javaIncrement = this.javaIncrement(javaIncrement);
                }
            }
            return new BigInteger(javaIncrement, this.signum);
        }
    }
    
    int[] javaIncrement(int[] array) {
        int n = 0;
        for (int n2 = array.length - 1; n2 >= 0 && n == 0; n = ++array[n2], --n2) {}
        if (n == 0) {
            array = new int[array.length + 1];
            array[0] = 1;
        }
        return array;
    }
    
    public BigInteger and(final BigInteger bigInteger) {
        final int[] array = new int[Math.max(this.intLength(), bigInteger.intLength())];
        for (int i = 0; i < array.length; ++i) {
            array[i] = (this.getInt(array.length - i - 1) & bigInteger.getInt(array.length - i - 1));
        }
        return valueOf(array);
    }
    
    public BigInteger or(final BigInteger bigInteger) {
        final int[] array = new int[Math.max(this.intLength(), bigInteger.intLength())];
        for (int i = 0; i < array.length; ++i) {
            array[i] = (this.getInt(array.length - i - 1) | bigInteger.getInt(array.length - i - 1));
        }
        return valueOf(array);
    }
    
    public BigInteger xor(final BigInteger bigInteger) {
        final int[] array = new int[Math.max(this.intLength(), bigInteger.intLength())];
        for (int i = 0; i < array.length; ++i) {
            array[i] = (this.getInt(array.length - i - 1) ^ bigInteger.getInt(array.length - i - 1));
        }
        return valueOf(array);
    }
    
    public BigInteger not() {
        final int[] array = new int[this.intLength()];
        for (int i = 0; i < array.length; ++i) {
            array[i] = (this.getInt(array.length - i - 1) ^ -1);
        }
        return valueOf(array);
    }
    
    public BigInteger andNot(final BigInteger bigInteger) {
        final int[] array = new int[Math.max(this.intLength(), bigInteger.intLength())];
        for (int i = 0; i < array.length; ++i) {
            array[i] = (this.getInt(array.length - i - 1) & (bigInteger.getInt(array.length - i - 1) ^ -1));
        }
        return valueOf(array);
    }
    
    public boolean testBit(final int n) {
        if (n < 0) {
            throw new ArithmeticException("Negative bit address");
        }
        return (this.getInt(n >>> 5) & 1 << (n & 0x1F)) != 0x0;
    }
    
    public BigInteger setBit(final int n) {
        if (n < 0) {
            throw new ArithmeticException("Negative bit address");
        }
        final int n2 = n >>> 5;
        final int[] array = new int[Math.max(this.intLength(), n2 + 2)];
        for (int i = 0; i < array.length; ++i) {
            array[array.length - i - 1] = this.getInt(i);
        }
        final int[] array2 = array;
        final int n3 = array.length - n2 - 1;
        array2[n3] |= 1 << (n & 0x1F);
        return valueOf(array);
    }
    
    public BigInteger clearBit(final int n) {
        if (n < 0) {
            throw new ArithmeticException("Negative bit address");
        }
        final int n2 = n >>> 5;
        final int[] array = new int[Math.max(this.intLength(), (n + 1 >>> 5) + 1)];
        for (int i = 0; i < array.length; ++i) {
            array[array.length - i - 1] = this.getInt(i);
        }
        final int[] array2 = array;
        final int n3 = array.length - n2 - 1;
        array2[n3] &= (1 << (n & 0x1F) ^ -1);
        return valueOf(array);
    }
    
    public BigInteger flipBit(final int n) {
        if (n < 0) {
            throw new ArithmeticException("Negative bit address");
        }
        final int n2 = n >>> 5;
        final int[] array = new int[Math.max(this.intLength(), n2 + 2)];
        for (int i = 0; i < array.length; ++i) {
            array[array.length - i - 1] = this.getInt(i);
        }
        final int[] array2 = array;
        final int n3 = array.length - n2 - 1;
        array2[n3] ^= 1 << (n & 0x1F);
        return valueOf(array);
    }
    
    public int getLowestSetBit() {
        int n = this.lowestSetBit - 2;
        if (n == -2) {
            n = 0;
            if (this.signum == 0) {
                --n;
            }
            else {
                int n2;
                int int1;
                for (n2 = 0; (int1 = this.getInt(n2)) == 0; ++n2) {}
                n += (n2 << 5) + Integer.numberOfTrailingZeros(int1);
            }
            this.lowestSetBit = n + 2;
        }
        return n;
    }
    
    public int bitLength() {
        int n = this.bitLength - 1;
        if (n == -1) {
            final int length = this.mag.length;
            if (length == 0) {
                n = 0;
            }
            else {
                final int n2 = (length - 1 << 5) + bitLengthForInt(this.mag[0]);
                if (this.signum < 0) {
                    boolean b = Integer.bitCount(this.mag[0]) == 1;
                    for (int n3 = 1; n3 < length && b; b = (this.mag[n3] == 0), ++n3) {}
                    n = (b ? (n2 - 1) : n2);
                }
                else {
                    n = n2;
                }
            }
            this.bitLength = n + 1;
        }
        return n;
    }
    
    public int bitCount() {
        int n = this.bitCount - 1;
        if (n == -1) {
            n = 0;
            for (int i = 0; i < this.mag.length; ++i) {
                n += Integer.bitCount(this.mag[i]);
            }
            if (this.signum < 0) {
                int n2 = 0;
                int n3;
                for (n3 = this.mag.length - 1; this.mag[n3] == 0; --n3) {
                    n2 += 32;
                }
                n += n2 + Integer.numberOfTrailingZeros(this.mag[n3]) - 1;
            }
            this.bitCount = n + 1;
        }
        return n;
    }
    
    public boolean isProbablePrime(final int n) {
        if (n <= 0) {
            return true;
        }
        final BigInteger abs = this.abs();
        return abs.equals(BigInteger.TWO) || (abs.testBit(0) && !abs.equals(BigInteger.ONE) && abs.primeToCertainty(n, null));
    }
    
    public int compareTo(final BigInteger bigInteger) {
        if (this.signum != bigInteger.signum) {
            return (this.signum > bigInteger.signum) ? 1 : -1;
        }
        switch (this.signum) {
            case 1: {
                return this.compareMagnitude(bigInteger);
            }
            case -1: {
                return bigInteger.compareMagnitude(this);
            }
            default: {
                return 0;
            }
        }
    }
    
    final int compareMagnitude(final BigInteger bigInteger) {
        final int[] mag = this.mag;
        final int length = mag.length;
        final int[] mag2 = bigInteger.mag;
        final int length2 = mag2.length;
        if (length < length2) {
            return -1;
        }
        if (length > length2) {
            return 1;
        }
        for (int i = 0; i < length; ++i) {
            final int n = mag[i];
            final int n2 = mag2[i];
            if (n != n2) {
                return ((n & 0xFFFFFFFFL) < (n2 & 0xFFFFFFFFL)) ? -1 : 1;
            }
        }
        return 0;
    }
    
    public boolean equals(final Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof BigInteger)) {
            return false;
        }
        final BigInteger bigInteger = (BigInteger)o;
        if (bigInteger.signum != this.signum) {
            return false;
        }
        final int[] mag = this.mag;
        final int length = mag.length;
        final int[] mag2 = bigInteger.mag;
        if (length != mag2.length) {
            return false;
        }
        for (int i = 0; i < length; ++i) {
            if (mag2[i] != mag[i]) {
                return false;
            }
        }
        return true;
    }
    
    public BigInteger min(final BigInteger bigInteger) {
        return (this.compareTo(bigInteger) < 0) ? this : bigInteger;
    }
    
    public BigInteger max(final BigInteger bigInteger) {
        return (this.compareTo(bigInteger) > 0) ? this : bigInteger;
    }
    
    public int hashCode() {
        int n = 0;
        for (int i = 0; i < this.mag.length; ++i) {
            n = (int)(31 * n + (this.mag[i] & 0xFFFFFFFFL));
        }
        return n * this.signum;
    }
    
    public String toString(int n) {
        if (this.signum == 0) {
            return "0";
        }
        if (n < 2 || n > 36) {
            n = 10;
        }
        final String[] array = new String[(4 * this.mag.length + 6) / 7];
        BigInteger abs = this.abs();
        int n2 = 0;
        while (abs.signum != 0) {
            final BigInteger bigInteger = BigInteger.longRadix[n];
            final MutableBigInteger mutableBigInteger = new MutableBigInteger();
            final MutableBigInteger divide = new MutableBigInteger(abs.mag).divide(new MutableBigInteger(bigInteger.mag), mutableBigInteger);
            final BigInteger bigInteger2 = mutableBigInteger.toBigInteger(abs.signum * bigInteger.signum);
            array[n2++] = Long.toString(divide.toBigInteger(abs.signum * bigInteger.signum).longValue(), n);
            abs = bigInteger2;
        }
        final StringBuilder sb = new StringBuilder(n2 * BigInteger.digitsPerLong[n] + 1);
        if (this.signum < 0) {
            sb.append('-');
        }
        sb.append(array[n2 - 1]);
        for (int i = n2 - 2; i >= 0; --i) {
            final int n3 = BigInteger.digitsPerLong[n] - array[i].length();
            if (n3 != 0) {
                sb.append(BigInteger.zeros[n3]);
            }
            sb.append(array[i]);
        }
        return sb.toString();
    }
    
    public String toString() {
        return this.toString(10);
    }
    
    public byte[] toByteArray() {
        final int n = this.bitLength() / 8 + 1;
        final byte[] array = new byte[n];
        int i = n - 1;
        int n2 = 4;
        int int1 = 0;
        int n3 = 0;
        while (i >= 0) {
            if (n2 == 4) {
                int1 = this.getInt(n3++);
                n2 = 1;
            }
            else {
                int1 >>>= 8;
                ++n2;
            }
            array[i] = (byte)int1;
            --i;
        }
        return array;
    }
    
    public int intValue() {
        return this.getInt(0);
    }
    
    public long longValue() {
        long n = 0L;
        for (int i = 1; i >= 0; --i) {
            n = (n << 32) + (this.getInt(i) & 0xFFFFFFFFL);
        }
        return n;
    }
    
    public float floatValue() {
        return Float.parseFloat(this.toString());
    }
    
    public double doubleValue() {
        return Double.parseDouble(this.toString());
    }
    
    private static int[] stripLeadingZeroInts(final int[] array) {
        int length;
        int n;
        for (length = array.length, n = 0; n < length && array[n] == 0; ++n) {}
        return Arrays.copyOfRange(array, n, length);
    }
    
    private static int[] trustedStripLeadingZeroInts(final int[] array) {
        int length;
        int n;
        for (length = array.length, n = 0; n < length && array[n] == 0; ++n) {}
        return (n == 0) ? array : Arrays.copyOfRange(array, n, length);
    }
    
    private static int[] stripLeadingZeroBytes(final byte[] array) {
        int length;
        int n;
        for (length = array.length, n = 0; n < length && array[n] == 0; ++n) {}
        final int n2 = length - n + 3 >>> 2;
        final int[] array2 = new int[n2];
        int n3 = length - 1;
        for (int i = n2 - 1; i >= 0; --i) {
            array2[i] = (array[n3--] & 0xFF);
            for (int min = Math.min(3, n3 - n + 1), j = 8; j <= min << 3; j += 8) {
                final int[] array3 = array2;
                final int n4 = i;
                array3[n4] |= (array[n3--] & 0xFF) << j;
            }
        }
        return array2;
    }
    
    private static int[] makePositive(final byte[] array) {
        int length;
        int n;
        for (length = array.length, n = 0; n < length && array[n] == -1; ++n) {}
        int n2;
        for (n2 = n; n2 < length && array[n2] == 0; ++n2) {}
        final int n3 = (length - n + ((n2 == length) ? 1 : 0) + 3) / 4;
        final int[] array2 = new int[n3];
        int n4 = length - 1;
        for (int i = n3 - 1; i >= 0; --i) {
            array2[i] = (array[n4--] & 0xFF);
            int min = Math.min(3, n4 - n + 1);
            if (min < 0) {
                min = 0;
            }
            for (int j = 8; j <= 8 * min; j += 8) {
                final int[] array3 = array2;
                final int n5 = i;
                array3[n5] |= (array[n4--] & 0xFF) << j;
            }
            array2[i] = ((array2[i] ^ -1) & -1 >>> 8 * (3 - min));
        }
        for (int k = array2.length - 1; k >= 0; --k) {
            array2[k] = (int)((array2[k] & 0xFFFFFFFFL) + 1L);
            if (array2[k] != 0) {
                break;
            }
        }
        return array2;
    }
    
    private static int[] makePositive(final int[] array) {
        int n;
        for (n = 0; n < array.length && array[n] == -1; ++n) {}
        int n2;
        for (n2 = n; n2 < array.length && array[n2] == 0; ++n2) {}
        final boolean b = n2 == array.length;
        final int[] array2 = new int[array.length - n + (b ? 1 : 0)];
        for (int i = n; i < array.length; ++i) {
            array2[i - n + (b ? 1 : 0)] = (array[i] ^ -1);
        }
        for (int n3 = array2.length - 1; ++array2[n3] == 0; --n3) {}
        return array2;
    }
    
    private int intLength() {
        return (this.bitLength() >>> 5) + 1;
    }
    
    private int signBit() {
        return (this.signum < 0) ? 1 : 0;
    }
    
    private int signInt() {
        return (this.signum < 0) ? -1 : 0;
    }
    
    private int getInt(final int n) {
        if (n < 0) {
            return 0;
        }
        if (n >= this.mag.length) {
            return this.signInt();
        }
        final int n2 = this.mag[this.mag.length - n - 1];
        return (this.signum >= 0) ? n2 : ((n <= this.firstNonzeroIntNum()) ? (-n2) : (n2 ^ -1));
    }
    
    private int firstNonzeroIntNum() {
        int n = this.firstNonzeroIntNum - 2;
        if (n == -2) {
            final int length = this.mag.length;
            int n2;
            for (n2 = length - 1; n2 >= 0 && this.mag[n2] == 0; --n2) {}
            n = length - n2 - 1;
            this.firstNonzeroIntNum = n + 2;
        }
        return n;
    }
    
    private void readObject(final ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        final ObjectInputStream.GetField readFields = objectInputStream.readFields();
        final int value = readFields.get("signum", -2);
        final byte[] array = (byte[])readFields.get("magnitude", (Object)null);
        if (value < -1 || value > 1) {
            String s = "BigInteger: Invalid signum value";
            if (readFields.defaulted("signum")) {
                s = "BigInteger: Signum not present in stream";
            }
            throw new StreamCorruptedException(s);
        }
        if (array.length == 0 != (value == 0)) {
            String s2 = "BigInteger: signum-magnitude mismatch";
            if (readFields.defaulted("magnitude")) {
                s2 = "BigInteger: Magnitude not present in stream";
            }
            throw new StreamCorruptedException(s2);
        }
        BigInteger.unsafe.putIntVolatile(this, BigInteger.signumOffset, value);
        BigInteger.unsafe.putObjectVolatile(this, BigInteger.magOffset, stripLeadingZeroBytes(array));
    }
    
    private void writeObject(final ObjectOutputStream objectOutputStream) throws IOException {
        final ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
        putFields.put("signum", this.signum);
        putFields.put("magnitude", this.magSerializedForm());
        putFields.put("bitCount", -1);
        putFields.put("bitLength", -1);
        putFields.put("lowestSetBit", -2);
        putFields.put("firstNonzeroByteNum", -2);
        objectOutputStream.writeFields();
    }
    
    private byte[] magSerializedForm() {
        final int length = this.mag.length;
        final int n = ((length == 0) ? 0 : ((length - 1 << 5) + bitLengthForInt(this.mag[0]))) + 7 >>> 3;
        final byte[] array = new byte[n];
        int i = n - 1;
        int n2 = 4;
        int n3 = length - 1;
        int n4 = 0;
        while (i >= 0) {
            if (n2 == 4) {
                n4 = this.mag[n3--];
                n2 = 1;
            }
            else {
                n4 >>>= 8;
                ++n2;
            }
            array[i] = (byte)n4;
            --i;
        }
        return array;
    }
    
    static {
        BigInteger.bitsPerDigit = new long[] { 0L, 0L, 1024L, 1624L, 2048L, 2378L, 2648L, 2875L, 3072L, 3247L, 3402L, 3543L, 3672L, 3790L, 3899L, 4001L, 4096L, 4186L, 4271L, 4350L, 4426L, 4498L, 4567L, 4633L, 4696L, 4756L, 4814L, 4870L, 4923L, 4975L, 5025L, 5074L, 5120L, 5166L, 5210L, 5253L, 5295L };
        SMALL_PRIME_PRODUCT = valueOf(152125131763605L);
        BigInteger.posConst = new BigInteger[17];
        BigInteger.negConst = new BigInteger[17];
        for (int i = 1; i <= 16; ++i) {
            final int[] array = { i };
            BigInteger.posConst[i] = new BigInteger(array, 1);
            BigInteger.negConst[i] = new BigInteger(array, -1);
        }
        ZERO = new BigInteger(new int[0], 0);
        ONE = valueOf(1L);
        TWO = valueOf(2L);
        TEN = valueOf(10L);
        BigInteger.bnExpModThreshTable = new int[] { 7, 25, 81, 241, 673, 1793, Integer.MAX_VALUE };
        (BigInteger.zeros = new String[64])[63] = "000000000000000000000000000000000000000000000000000000000000000";
        for (int j = 0; j < 63; ++j) {
            BigInteger.zeros[j] = BigInteger.zeros[63].substring(0, j);
        }
        BigInteger.digitsPerLong = new int[] { 0, 0, 62, 39, 31, 27, 24, 22, 20, 19, 18, 18, 17, 17, 16, 16, 15, 15, 15, 14, 14, 14, 14, 13, 13, 13, 13, 13, 13, 12, 12, 12, 12, 12, 12, 12, 12 };
        BigInteger.longRadix = new BigInteger[] { null, null, valueOf(4611686018427387904L), valueOf(4052555153018976267L), valueOf(4611686018427387904L), valueOf(7450580596923828125L), valueOf(4738381338321616896L), valueOf(3909821048582988049L), valueOf(1152921504606846976L), valueOf(1350851717672992089L), valueOf(1000000000000000000L), valueOf(5559917313492231481L), valueOf(2218611106740436992L), valueOf(8650415919381337933L), valueOf(2177953337809371136L), valueOf(6568408355712890625L), valueOf(1152921504606846976L), valueOf(2862423051509815793L), valueOf(6746640616477458432L), valueOf(799006685782884121L), valueOf(1638400000000000000L), valueOf(3243919932521508681L), valueOf(6221821273427820544L), valueOf(504036361936467383L), valueOf(876488338465357824L), valueOf(1490116119384765625L), valueOf(2481152873203736576L), valueOf(4052555153018976267L), valueOf(6502111422497947648L), valueOf(353814783205469041L), valueOf(531441000000000000L), valueOf(787662783788549761L), valueOf(1152921504606846976L), valueOf(1667889514952984961L), valueOf(2386420683693101056L), valueOf(3379220508056640625L), valueOf(4738381338321616896L) };
        BigInteger.digitsPerInt = new int[] { 0, 0, 30, 19, 15, 13, 11, 11, 10, 9, 9, 8, 8, 8, 8, 7, 7, 7, 7, 7, 7, 7, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5 };
        BigInteger.intRadix = new int[] { 0, 0, 1073741824, 1162261467, 1073741824, 1220703125, 362797056, 1977326743, 1073741824, 387420489, 1000000000, 214358881, 429981696, 815730721, 1475789056, 170859375, 268435456, 410338673, 612220032, 893871739, 1280000000, 1801088541, 113379904, 148035889, 191102976, 244140625, 308915776, 387420489, 481890304, 594823321, 729000000, 887503681, 1073741824, 1291467969, 1544804416, 1838265625, 60466176 };
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("signum", Integer.TYPE), new ObjectStreamField("magnitude", byte[].class), new ObjectStreamField("bitCount", Integer.TYPE), new ObjectStreamField("bitLength", Integer.TYPE), new ObjectStreamField("firstNonzeroByteNum", Integer.TYPE), new ObjectStreamField("lowestSetBit", Integer.TYPE) };
        unsafe = Unsafe.getUnsafe();
        try {
            signumOffset = BigInteger.unsafe.objectFieldOffset(BigInteger.class.getDeclaredField("signum"));
            magOffset = BigInteger.unsafe.objectFieldOffset(BigInteger.class.getDeclaredField("mag"));
        }
        catch (Exception ex) {
            throw new Error(ex);
        }
    }
}
