package com.calm.android.services;

import android.media.*;
import com.calm.android.data.*;
import android.support.v4.app.*;
import com.calm.android.activities.*;
import android.content.*;
import android.app.*;
import android.util.*;
import java.io.*;
import android.widget.*;
import android.os.*;

public class AudioService extends Service implements MediaPlayer$OnCompletionListener, MediaPlayer$OnPreparedListener, MediaPlayer$OnErrorListener
{
    public static final String ACTION_IS_STOPPED = "com.calm.android.action.IS_STOPPED";
    public static final String ACTION_PAUSE = "com.calm.android.action.PAUSE";
    public static final String ACTION_PLAY = "com.calm.android.action.PLAY";
    public static final String ACTION_STOP = "com.calm.android.action.STOP";
    public static final String ACTION_URL = "com.calm.android.action.URL";
    public static final String BROADCAST_ACTION = "com.calm.android.BROADCAST_AUDIO";
    public static final String BROADCAST_CATEGORY = "com.calm.android.BROADCAST_AUDIO_CATEGORY";
    public static final String BROADCAST_PLAYBACK_DURATION = "broadcast_duration";
    public static final String BROADCAST_PLAYBACK_POSITION = "broadcast_position";
    public static final String BROADCAST_PLAYBACK_STATUS = "broadcast_status";
    public static final float DUCK_VOLUME = 0.1f;
    public static final String EXTRAS_AUDIO_PATH = "audio_path";
    public static final int SESSION_NOTIFICATION_ID = 42;
    static final String TAG;
    private static AudioStatus mLastBroadcastStatus;
    AudioFocus mAudioFocus;
    private String mCurrentSongPath;
    private float mCurrentVolume;
    MediaPlayer mPlayer;
    private State mState;
    
    static {
        TAG = AudioService.class.getSimpleName();
        AudioService.mLastBroadcastStatus = AudioStatus.Stopped;
    }
    
    public AudioService() {
        this.mCurrentVolume = 1.0f;
        this.mPlayer = null;
        this.mCurrentSongPath = null;
        this.mState = State.Stopped;
        this.mAudioFocus = AudioFocus.Focused;
    }
    
    private void configAndStartMediaPlayer() {
        if (this.mAudioFocus == AudioFocus.NoFocusNoDuck) {
            if (this.mPlayer.isPlaying()) {
                this.mPlayer.pause();
            }
        }
        else {
            if (this.mAudioFocus == AudioFocus.NoFocusCanDuck) {
                this.mCurrentVolume = 0.1f;
                this.mPlayer.setVolume(0.1f, 0.1f);
            }
            else {
                this.mCurrentVolume = 1.0f;
                this.mPlayer.setVolume(1.0f, 1.0f);
            }
            if (!this.mPlayer.isPlaying()) {
                this.mPlayer.start();
                this.broadcastStatus(AudioStatus.Playing);
            }
        }
    }
    
    private void createMediaPlayerIfNeeded() {
        if (this.mPlayer == null) {
            (this.mPlayer = new MediaPlayer()).setWakeMode(this.getApplicationContext(), 1);
            this.mPlayer.setOnPreparedListener((MediaPlayer$OnPreparedListener)this);
            this.mPlayer.setOnCompletionListener((MediaPlayer$OnCompletionListener)this);
            this.mPlayer.setOnErrorListener((MediaPlayer$OnErrorListener)this);
            return;
        }
        this.mPlayer.reset();
    }
    
    public static AudioStatus getAudioStatus() {
        return AudioService.mLastBroadcastStatus;
    }
    
    public static Notification getSessionNotification(final Context context) {
        return getSessionNotification(context, null);
    }
    
    public static Notification getSessionNotification(final Context context, final Guide guide) {
        final NotificationCompat$Builder setContentIntent = new NotificationCompat$Builder(context).setSmallIcon(2130837696).setWhen(System.currentTimeMillis()).setContentIntent(PendingIntent.getActivity(context, 42, new Intent(context, (Class)MainActivity.class), 268435456));
        if (guide != null) {
            setContentIntent.setContentTitle((CharSequence)"Session running");
            setContentIntent.setContentText((CharSequence)guide.toString());
            setContentIntent.setPriority(1);
        }
        else {
            setContentIntent.setContentTitle((CharSequence)"Calm running");
        }
        final Notification build = setContentIntent.build();
        build.defaults = 0;
        build.flags |= 0x20;
        return build;
    }
    
    private void playNextSong(final Intent p0) {
        // 
        // This method could not be decompiled.
        // 
        // Original Bytecode:
        // 
        //     0: aload_1        
        //     1: ifnonnull       5
        //     4: return         
        //     5: aload_1        
        //     6: ldc             "audio_path"
        //     8: invokevirtual   android/content/Intent.getStringExtra:(Ljava/lang/String;)Ljava/lang/String;
        //    11: astore_2       
        //    12: aload_0        
        //    13: getfield        com/calm/android/services/AudioService.mState:Lcom/calm/android/services/AudioService$State;
        //    16: getstatic       com/calm/android/services/AudioService$State.Playing:Lcom/calm/android/services/AudioService$State;
        //    19: if_acmpne       37
        //    22: aload_2        
        //    23: ifnull          4
        //    26: aload_2        
        //    27: aload_0        
        //    28: getfield        com/calm/android/services/AudioService.mCurrentSongPath:Ljava/lang/String;
        //    31: invokevirtual   java/lang/String.equals:(Ljava/lang/Object;)Z
        //    34: ifne            4
        //    37: aload_2        
        //    38: aload_0        
        //    39: getfield        com/calm/android/services/AudioService.mCurrentSongPath:Ljava/lang/String;
        //    42: invokevirtual   java/lang/String.equals:(Ljava/lang/Object;)Z
        //    45: ifeq            63
        //    48: aload_0        
        //    49: getfield        com/calm/android/services/AudioService.mState:Lcom/calm/android/services/AudioService$State;
        //    52: getstatic       com/calm/android/services/AudioService$State.Paused:Lcom/calm/android/services/AudioService$State;
        //    55: if_acmpne       63
        //    58: aload_0        
        //    59: invokespecial   com/calm/android/services/AudioService.processPlayRequest:()V
        //    62: return         
        //    63: aload_0        
        //    64: getstatic       com/calm/android/services/AudioService$State.Stopped:Lcom/calm/android/services/AudioService$State;
        //    67: putfield        com/calm/android/services/AudioService.mState:Lcom/calm/android/services/AudioService$State;
        //    70: aload_0        
        //    71: iconst_0       
        //    72: invokespecial   com/calm/android/services/AudioService.relaxResources:(Z)V
        //    75: getstatic       com/calm/android/services/AudioService.TAG:Ljava/lang/String;
        //    78: new             Ljava/lang/StringBuilder;
        //    81: dup            
        //    82: invokespecial   java/lang/StringBuilder.<init>:()V
        //    85: ldc_w           "Starting "
        //    88: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/String;)Ljava/lang/StringBuilder;
        //    91: aload_2        
        //    92: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/String;)Ljava/lang/StringBuilder;
        //    95: invokevirtual   java/lang/StringBuilder.toString:()Ljava/lang/String;
        //    98: invokestatic    android/util/Log.v:(Ljava/lang/String;Ljava/lang/String;)I
        //   101: pop            
        //   102: aload_0        
        //   103: invokespecial   com/calm/android/services/AudioService.createMediaPlayerIfNeeded:()V
        //   106: aload_0        
        //   107: getfield        com/calm/android/services/AudioService.mPlayer:Landroid/media/MediaPlayer;
        //   110: iconst_3       
        //   111: invokevirtual   android/media/MediaPlayer.setAudioStreamType:(I)V
        //   114: aload_2        
        //   115: invokestatic    android/net/Uri.parse:(Ljava/lang/String;)Landroid/net/Uri;
        //   118: astore          6
        //   120: aload_0        
        //   121: getfield        com/calm/android/services/AudioService.mPlayer:Landroid/media/MediaPlayer;
        //   124: aload_0        
        //   125: invokevirtual   com/calm/android/services/AudioService.getBaseContext:()Landroid/content/Context;
        //   128: aload           6
        //   130: invokevirtual   android/media/MediaPlayer.setDataSource:(Landroid/content/Context;Landroid/net/Uri;)V
        //   133: aload_0        
        //   134: getfield        com/calm/android/services/AudioService.mPlayer:Landroid/media/MediaPlayer;
        //   137: invokevirtual   android/media/MediaPlayer.prepareAsync:()V
        //   140: aload_0        
        //   141: aload_2        
        //   142: putfield        com/calm/android/services/AudioService.mCurrentSongPath:Ljava/lang/String;
        //   145: return         
        //   146: astore_3       
        //   147: ldc_w           "MusicService"
        //   150: new             Ljava/lang/StringBuilder;
        //   153: dup            
        //   154: invokespecial   java/lang/StringBuilder.<init>:()V
        //   157: ldc_w           "IOException playing next song: "
        //   160: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/String;)Ljava/lang/StringBuilder;
        //   163: aload_3        
        //   164: invokevirtual   java/io/IOException.getMessage:()Ljava/lang/String;
        //   167: invokevirtual   java/lang/StringBuilder.append:(Ljava/lang/String;)Ljava/lang/StringBuilder;
        //   170: invokevirtual   java/lang/StringBuilder.toString:()Ljava/lang/String;
        //   173: invokestatic    android/util/Log.e:(Ljava/lang/String;Ljava/lang/String;)I
        //   176: pop            
        //   177: aload_3        
        //   178: invokevirtual   java/io/IOException.printStackTrace:()V
        //   181: return         
        //   182: astore          7
        //   184: aload_0        
        //   185: getfield        com/calm/android/services/AudioService.mPlayer:Landroid/media/MediaPlayer;
        //   188: invokevirtual   android/media/MediaPlayer.reset:()V
        //   191: aload_0        
        //   192: getfield        com/calm/android/services/AudioService.mPlayer:Landroid/media/MediaPlayer;
        //   195: aload_0        
        //   196: invokevirtual   com/calm/android/services/AudioService.getBaseContext:()Landroid/content/Context;
        //   199: aload           6
        //   201: invokevirtual   android/media/MediaPlayer.setDataSource:(Landroid/content/Context;Landroid/net/Uri;)V
        //   204: goto            133
        //    Exceptions:
        //  Try           Handler
        //  Start  End    Start  End    Type                             
        //  -----  -----  -----  -----  ---------------------------------
        //  75     120    146    182    Ljava/io/IOException;
        //  120    133    182    207    Ljava/lang/IllegalStateException;
        //  120    133    146    182    Ljava/io/IOException;
        //  133    145    146    182    Ljava/io/IOException;
        //  184    204    146    182    Ljava/io/IOException;
        // 
        // The error that occurred was:
        // 
        // java.lang.IllegalStateException: Expression is linked from several locations: Label_0133:
        //     at com.strobel.decompiler.ast.Error.expressionLinkedFromMultipleLocations(Error.java:27)
        //     at com.strobel.decompiler.ast.AstOptimizer.mergeDisparateObjectInitializations(AstOptimizer.java:2592)
        //     at com.strobel.decompiler.ast.AstOptimizer.optimize(AstOptimizer.java:235)
        //     at com.strobel.decompiler.ast.AstOptimizer.optimize(AstOptimizer.java:42)
        //     at com.strobel.decompiler.languages.java.ast.AstMethodBodyBuilder.createMethodBody(AstMethodBodyBuilder.java:214)
        //     at com.strobel.decompiler.languages.java.ast.AstMethodBodyBuilder.createMethodBody(AstMethodBodyBuilder.java:99)
        //     at com.strobel.decompiler.languages.java.ast.AstBuilder.createMethodBody(AstBuilder.java:757)
        //     at com.strobel.decompiler.languages.java.ast.AstBuilder.createMethod(AstBuilder.java:655)
        //     at com.strobel.decompiler.languages.java.ast.AstBuilder.addTypeMembers(AstBuilder.java:532)
        //     at com.strobel.decompiler.languages.java.ast.AstBuilder.createTypeCore(AstBuilder.java:499)
        //     at com.strobel.decompiler.languages.java.ast.AstBuilder.createTypeNoCache(AstBuilder.java:141)
        //     at com.strobel.decompiler.languages.java.ast.AstBuilder.createType(AstBuilder.java:130)
        //     at com.strobel.decompiler.languages.java.ast.AstBuilder.addType(AstBuilder.java:105)
        //     at com.strobel.decompiler.languages.java.JavaLanguage.buildAst(JavaLanguage.java:71)
        //     at com.strobel.decompiler.languages.java.JavaLanguage.decompileType(JavaLanguage.java:59)
        //     at com.strobel.decompiler.DecompilerDriver.decompileType(DecompilerDriver.java:304)
        //     at com.strobel.decompiler.DecompilerDriver.decompileJar(DecompilerDriver.java:225)
        //     at com.strobel.decompiler.DecompilerDriver.main(DecompilerDriver.java:125)
        // 
        throw new IllegalStateException("An error occurred while decompiling this method.");
    }
    
    private void processAddRequest(final Intent intent) {
        if (this.mState == State.Playing || this.mState == State.Paused || this.mState == State.Stopped) {
            this.playNextSong(intent);
            this.startForeground(42, getSessionNotification(this.getBaseContext()));
        }
    }
    
    private void processIsStopped() {
        if (this.mState == State.Stopped) {
            this.broadcastStatus(AudioStatus.Stopped);
        }
    }
    
    private void processPauseRequest() {
        if (this.mState == State.Playing) {
            this.broadcastStatus(AudioStatus.Paused);
            this.mState = State.Paused;
            new Fader().fade(this.mPlayer, this.mCurrentVolume, 500, new Runnable() {
                @Override
                public void run() {
                    AudioService.this.mPlayer.pause();
                    AudioService.this.relaxResources(false);
                }
            });
        }
    }
    
    private void processPlayRequest() {
        if (this.mState == State.Paused) {
            this.mState = State.Playing;
            this.configAndStartMediaPlayer();
        }
    }
    
    private void processStopRequest() {
        this.processStopRequest(false);
        this.broadcastStatus(AudioStatus.Stopped);
        this.mCurrentSongPath = null;
        this.stopForeground(true);
    }
    
    private void processStopRequest(final boolean b) {
        if (this.mState == State.Playing || this.mState == State.Paused || b) {
            this.mState = State.Stopped;
            new Fader().fade(this.mPlayer, this.mCurrentVolume, 500, new Runnable() {
                @Override
                public void run() {
                    AudioService.this.relaxResources(true);
                    AudioService.this.stopSelf();
                }
            });
        }
    }
    
    private void relaxResources(final boolean b) {
        this.stopForeground(true);
        if (b && this.mPlayer != null) {
            this.mPlayer.reset();
            this.mPlayer.release();
            this.mPlayer = null;
        }
    }
    
    protected void broadcastStatus(final AudioStatus mLastBroadcastStatus) {
        Log.v(AudioService.TAG, "broadcastStatus: " + mLastBroadcastStatus);
        AudioService.mLastBroadcastStatus = mLastBroadcastStatus;
        final Intent intent = new Intent();
        intent.setAction("com.calm.android.BROADCAST_AUDIO");
        intent.addCategory("com.calm.android.BROADCAST_AUDIO_CATEGORY");
        intent.putExtra("broadcast_status", (Serializable)mLastBroadcastStatus);
        if (this.mPlayer != null && this.mPlayer.isPlaying()) {
            intent.putExtra("broadcast_duration", this.mPlayer.getDuration());
            intent.putExtra("broadcast_position", this.mPlayer.getCurrentPosition());
        }
        this.sendBroadcast(intent);
    }
    
    protected boolean getIsLooping() {
        return false;
    }
    
    public IBinder onBind(final Intent intent) {
        return null;
    }
    
    public void onCompletion(final MediaPlayer mediaPlayer) {
        this.mCurrentSongPath = null;
        this.broadcastStatus(AudioStatus.Stopped);
    }
    
    public void onCreate() {
        Log.i(AudioService.TAG, "debug: Creating service");
    }
    
    public void onDestroy() {
        this.mState = State.Stopped;
        this.relaxResources(true);
    }
    
    public boolean onError(final MediaPlayer mediaPlayer, final int n, final int n2) {
        Toast.makeText(this.getApplicationContext(), (CharSequence)"Media player error! Resetting.", 0).show();
        Log.e(AudioService.TAG, "Error: what=" + String.valueOf(n) + ", extra=" + String.valueOf(n2));
        this.mState = State.Stopped;
        this.relaxResources(true);
        return true;
    }
    
    public void onPrepared(final MediaPlayer mediaPlayer) {
        this.mState = State.Playing;
        this.mPlayer.setLooping(this.getIsLooping());
        this.configAndStartMediaPlayer();
    }
    
    public int onStartCommand(final Intent intent, final int n, final int n2) {
        final String action = intent.getAction();
        Log.v(AudioService.TAG, "onStartCommand: " + action);
        if (action.equals("com.calm.android.action.PLAY")) {
            this.processPlayRequest();
        }
        else if (action.equals("com.calm.android.action.PAUSE")) {
            this.processPauseRequest();
        }
        else if (action.equals("com.calm.android.action.STOP")) {
            this.processStopRequest();
        }
        else if (action.equals("com.calm.android.action.URL")) {
            this.processAddRequest(intent);
        }
        else if (action.equals("com.calm.android.action.IS_STOPPED")) {
            this.processIsStopped();
        }
        return 2;
    }
    
    enum AudioFocus
    {
        Focused, 
        NoFocusCanDuck, 
        NoFocusNoDuck;
    }
    
    public enum AudioStatus
    {
        Paused, 
        Playing, 
        Stopped;
    }
    
    public static class Fader
    {
        private float mCurrentVolume;
        
        public Fader() {
            this.mCurrentVolume = 1.0f;
        }
        
        public void fade(final MediaPlayer mediaPlayer, final float mCurrentVolume, final int n, final Runnable runnable) {
            this.mCurrentVolume = mCurrentVolume;
            final Handler handler = new Handler();
            handler.post((Runnable)new Runnable() {
                @Override
                public void run() {
                    if (mediaPlayer == null) {
                        return;
                    }
                    Fader.this.mCurrentVolume -= 0.1f;
                    while (true) {
                        try {
                            mediaPlayer.setVolume(Fader.this.mCurrentVolume, Fader.this.mCurrentVolume);
                            if (Fader.this.mCurrentVolume <= 0.0f) {
                                Fader.this.mCurrentVolume = mCurrentVolume;
                                runnable.run();
                                return;
                            }
                        }
                        catch (IllegalStateException ex) {
                            Log.v(AudioService.TAG, "Error fading volume");
                            continue;
                        }
                        break;
                    }
                    handler.postDelayed((Runnable)this, (long)(n / 10));
                }
            });
        }
    }
    
    public enum State
    {
        Paused, 
        Playing, 
        Preparing, 
        Stopped;
    }
}
