package com.google.common.collect;

import com.google.common.annotations.GwtCompatible;
import com.google.common.annotations.GwtIncompatible;
import com.google.common.base.Objects;
import com.google.common.base.Preconditions;
import com.google.common.collect.BiMap;
import com.google.common.collect.ForwardingMap;
import com.google.common.collect.ForwardingMapEntry;
import com.google.common.collect.ForwardingSet;
import com.google.common.collect.Maps;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;
import javax.annotation.Nullable;

@GwtCompatible(
   emulated = true
)
abstract class AbstractBiMap<K extends Object, V extends Object> extends ForwardingMap<K, V> implements BiMap<K, V>, Serializable {

   @GwtIncompatible("Not needed in emulated source.")
   private static final long serialVersionUID;
   private transient Map<K, V> delegate;
   private transient Set<Entry<K, V>> entrySet;
   transient AbstractBiMap<V, K> inverse;
   private transient Set<K> keySet;
   private transient Set<V> valueSet;


   private AbstractBiMap(Map<K, V> var1, AbstractBiMap<V, K> var2) {
      this.delegate = var1;
      this.inverse = var2;
   }

   // $FF: synthetic method
   AbstractBiMap(Map var1, AbstractBiMap var2, Object var3) {
      this(var1, var2);
   }

   AbstractBiMap(Map<K, V> var1, Map<V, K> var2) {
      this.setDelegates(var1, var2);
   }

   private V putInBothMaps(@Nullable K var1, @Nullable V var2, boolean var3) {
      this.checkKey(var1);
      this.checkValue(var2);
      boolean var6 = this.containsKey(var1);
      if(var6 && Objects.equal(var2, this.get(var1))) {
         return var2;
      } else {
         if(var3) {
            this.inverse().remove(var2);
         } else {
            boolean var7;
            if(!this.containsValue(var2)) {
               var7 = true;
            } else {
               var7 = false;
            }

            Preconditions.checkArgument(var7, "value already present: %s", new Object[]{var2});
         }

         Object var8 = this.delegate.put(var1, var2);
         this.updateInverseMap(var1, var6, var8, var2);
         return var8;
      }
   }

   private V removeFromBothMaps(Object var1) {
      Object var2 = this.delegate.remove(var1);
      this.removeFromInverseMap(var2);
      return var2;
   }

   private void removeFromInverseMap(V var1) {
      this.inverse.delegate.remove(var1);
   }

   private void updateInverseMap(K var1, boolean var2, V var3, V var4) {
      if(var2) {
         this.removeFromInverseMap(var3);
      }

      this.inverse.delegate.put(var4, var1);
   }

   K checkKey(@Nullable K var1) {
      return var1;
   }

   V checkValue(@Nullable V var1) {
      return var1;
   }

   public void clear() {
      this.delegate.clear();
      this.inverse.delegate.clear();
   }

   public boolean containsValue(Object var1) {
      return this.inverse.containsKey(var1);
   }

   protected Map<K, V> delegate() {
      return this.delegate;
   }

   public Set<Entry<K, V>> entrySet() {
      Object var1 = this.entrySet;
      if(var1 == null) {
         var1 = new AbstractBiMap.EntrySet(null);
         this.entrySet = (Set)var1;
      }

      return (Set)var1;
   }

   public V forcePut(K var1, V var2) {
      return this.putInBothMaps(var1, var2, true);
   }

   public BiMap<V, K> inverse() {
      return this.inverse;
   }

   public Set<K> keySet() {
      Object var1 = this.keySet;
      if(var1 == null) {
         var1 = new AbstractBiMap.KeySet(null);
         this.keySet = (Set)var1;
      }

      return (Set)var1;
   }

   public V put(K var1, V var2) {
      return this.putInBothMaps(var1, var2, false);
   }

   public void putAll(Map<? extends K, ? extends V> var1) {
      Iterator var2 = var1.entrySet().iterator();

      while(var2.hasNext()) {
         Entry var3 = (Entry)var2.next();
         this.put(var3.getKey(), var3.getValue());
      }

   }

   public V remove(Object var1) {
      return this.containsKey(var1)?this.removeFromBothMaps(var1):null;
   }

   void setDelegates(Map<K, V> var1, Map<V, K> var2) {
      boolean var3 = true;
      boolean var4;
      if(this.delegate == null) {
         var4 = var3;
      } else {
         var4 = false;
      }

      Preconditions.checkState(var4);
      boolean var5;
      if(this.inverse == null) {
         var5 = var3;
      } else {
         var5 = false;
      }

      Preconditions.checkState(var5);
      Preconditions.checkArgument(var1.isEmpty());
      Preconditions.checkArgument(var2.isEmpty());
      if(var1 == var2) {
         var3 = false;
      }

      Preconditions.checkArgument(var3);
      this.delegate = var1;
      this.inverse = new AbstractBiMap.Inverse(var2, this, null);
   }

   void setInverse(AbstractBiMap<V, K> var1) {
      this.inverse = var1;
   }

   public Set<V> values() {
      Object var1 = this.valueSet;
      if(var1 == null) {
         var1 = new AbstractBiMap.ValueSet(null);
         this.valueSet = (Set)var1;
      }

      return (Set)var1;
   }

   private class EntrySet extends ForwardingSet<Entry<K, V>> {

      final Set<Entry<K, V>> esDelegate;


      private EntrySet() {
         this.esDelegate = AbstractBiMap.this.delegate.entrySet();
      }

      // $FF: synthetic method
      EntrySet(Object var2) {
         this();
      }

      public void clear() {
         AbstractBiMap.this.clear();
      }

      public boolean contains(Object var1) {
         return Maps.containsEntryImpl(this.delegate(), var1);
      }

      public boolean containsAll(Collection<?> var1) {
         return this.standardContainsAll(var1);
      }

      protected Set<Entry<K, V>> delegate() {
         return this.esDelegate;
      }

      public Iterator<Entry<K, V>> iterator() {
         return new Iterator() {

            Entry<K, V> entry;
            // $FF: synthetic field
            final Iterator val$iterator;

            {
               this.val$iterator = var2;
            }
            public boolean hasNext() {
               return this.val$iterator.hasNext();
            }
            public Entry<K, V> next() {
               this.entry = (Entry)this.val$iterator.next();
               return new ForwardingMapEntry() {

                  // $FF: synthetic field
                  final Entry val$finalEntry;

                  {
                     this.val$finalEntry = var2;
                  }
                  protected Entry<K, V> delegate() {
                     return this.val$finalEntry;
                  }
                  public V setValue(V var1) {
                     Preconditions.checkState(EntrySet.thisx.contains(this), "entry no longer in map");
                     if(Objects.equal(var1, this.getValue())) {
                        return var1;
                     } else {
                        boolean var2;
                        if(!AbstractBiMap.this.containsValue(var1)) {
                           var2 = true;
                        } else {
                           var2 = false;
                        }

                        Preconditions.checkArgument(var2, "value already present: %s", new Object[]{var1});
                        Object var3 = this.val$finalEntry.setValue(var1);
                        Preconditions.checkState(Objects.equal(var1, AbstractBiMap.this.get(this.getKey())), "entry no longer in map");
                        AbstractBiMap.this.updateInverseMap(this.getKey(), true, var3, var1);
                        return var3;
                     }
                  }
               };
            }
            public void remove() {
               boolean var1;
               if(this.entry != null) {
                  var1 = true;
               } else {
                  var1 = false;
               }

               Preconditions.checkState(var1);
               Object var2 = this.entry.getValue();
               this.val$iterator.remove();
               AbstractBiMap.this.removeFromInverseMap(var2);
            }
         };
      }

      public boolean remove(Object var1) {
         if(!this.esDelegate.contains(var1)) {
            return false;
         } else {
            Entry var2 = (Entry)var1;
            AbstractBiMap.this.inverse.delegate.remove(var2.getValue());
            this.esDelegate.remove(var2);
            return true;
         }
      }

      public boolean removeAll(Collection<?> var1) {
         return this.standardRemoveAll(var1);
      }

      public boolean retainAll(Collection<?> var1) {
         return this.standardRetainAll(var1);
      }

      public Object[] toArray() {
         return this.standardToArray();
      }

      public <T extends Object> T[] toArray(T[] var1) {
         return this.standardToArray(var1);
      }
   }

   private static class Inverse<K extends Object, V extends Object> extends AbstractBiMap<K, V> {

      @GwtIncompatible("Not needed in emulated source.")
      private static final long serialVersionUID;


      private Inverse(Map<K, V> var1, AbstractBiMap<V, K> var2) {
         super(var1, var2, null);
      }

      // $FF: synthetic method
      Inverse(Map var1, AbstractBiMap var2, Object var3) {
         this(var1, var2);
      }

      @GwtIncompatible("java.io.ObjectInputStream")
      private void readObject(ObjectInputStream var1) throws IOException, ClassNotFoundException {
         var1.defaultReadObject();
         this.setInverse((AbstractBiMap)var1.readObject());
      }

      @GwtIncompatible("java.io.ObjectOuputStream")
      private void writeObject(ObjectOutputStream var1) throws IOException {
         var1.defaultWriteObject();
         var1.writeObject(this.inverse());
      }

      K checkKey(K var1) {
         return this.inverse.checkValue(var1);
      }

      V checkValue(V var1) {
         return this.inverse.checkKey(var1);
      }

      @GwtIncompatible("Not needed in the emulated source.")
      Object readResolve() {
         return this.inverse().inverse();
      }
   }

   private class KeySet extends ForwardingSet<K> {

      private KeySet() {}

      // $FF: synthetic method
      KeySet(Object var2) {
         this();
      }

      public void clear() {
         AbstractBiMap.this.clear();
      }

      protected Set<K> delegate() {
         return AbstractBiMap.this.delegate.keySet();
      }

      public Iterator<K> iterator() {
         return Maps.keyIterator(AbstractBiMap.this.entrySet().iterator());
      }

      public boolean remove(Object var1) {
         if(!this.contains(var1)) {
            return false;
         } else {
            AbstractBiMap.this.removeFromBothMaps(var1);
            return true;
         }
      }

      public boolean removeAll(Collection<?> var1) {
         return this.standardRemoveAll(var1);
      }

      public boolean retainAll(Collection<?> var1) {
         return this.standardRetainAll(var1);
      }
   }

   private class ValueSet extends ForwardingSet<V> {

      final Set<V> valuesDelegate;


      private ValueSet() {
         this.valuesDelegate = AbstractBiMap.this.inverse.keySet();
      }

      // $FF: synthetic method
      ValueSet(Object var2) {
         this();
      }

      protected Set<V> delegate() {
         return this.valuesDelegate;
      }

      public Iterator<V> iterator() {
         return Maps.valueIterator(AbstractBiMap.this.entrySet().iterator());
      }

      public Object[] toArray() {
         return this.standardToArray();
      }

      public <T extends Object> T[] toArray(T[] var1) {
         return this.standardToArray(var1);
      }

      public String toString() {
         return this.standardToString();
      }
   }
}
